SystemOrganization addCategory: 'JSON'!

!Dictionary methodsFor: '*JSON-writing' stamp: 'tonyg 8/17/2005 00:33'!
jsonWriteOn: aStream
	| needComma |
	needComma _ false.
	aStream nextPut: ${.
	self keysAndValuesDo: [:k :v |
		needComma
			ifTrue: [ aStream nextPutAll: ', ' ]
			ifFalse: [ needComma _ true ].
		k asString jsonWriteOn: aStream.
		aStream nextPutAll: ': '.
		v jsonWriteOn: aStream.
	].
	aStream nextPut: $}.! !

TestCase subclass: #JsonTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'JSON'!

!JsonTests commentStamp: '<historical>' prior: 0!
I provide a number of test cases for class Json.!

!JsonTests methodsFor: 'as yet unclassified' stamp: 'tonyg 8/16/2005 23:37'!
json: aString equals: aValue
	self assert: (Json readFrom: aString readStream) = aValue.! !

!JsonTests methodsFor: 'as yet unclassified' stamp: 'tonyg 8/17/2005 16:24'!
render: anObject equals: aString
	self assert: (Json render: anObject) = aString! !

!JsonTests methodsFor: 'as yet unclassified' stamp: 'tonyg 8/16/2005 23:51'!
testArray
	self json: '[]' equals: #().
	self json: '[[]]' equals: #(#()).
	self json: '[[], []]' equals: #(#() #()).
	self json: '["hi", "there"]' equals: #('hi' 'there').
	self json: '[["a", "b", null]]' equals: #(('a' 'b' nil)).! !

!JsonTests methodsFor: 'as yet unclassified' stamp: 'tonyg 8/16/2005 23:42'!
testAtomFalse
	self json: 'false' equals: false.
	self json: '  false' equals: false.
	self json: 'false  ' equals: false.
	self json: '  false  ' equals: false.
! !

!JsonTests methodsFor: 'as yet unclassified' stamp: 'tonyg 8/16/2005 23:43'!
testAtomNull
	self json: 'null' equals: nil.
	self json: '  null' equals: nil.
	self json: 'null  ' equals: nil.
	self json: '  null  ' equals: nil.
! !

!JsonTests methodsFor: 'as yet unclassified' stamp: 'tonyg 8/17/2005 09:10'!
testAtomNumber
	self json: '1' equals: 1.
	self json: '0123' equals: 123.
	self json: '1.23e2' equals: 123.
	self json: '-1' equals: -1.
	self json: '-0' equals: 0.
	self json: '[-1.2]' equals: #(-1.2).! !

!JsonTests methodsFor: 'as yet unclassified' stamp: 'tonyg 8/16/2005 23:46'!
testAtomString
	self json: '"hi"' equals: 'hi'.
	self json: '"\""' equals: '"'.
	self json: '"\\"' equals: '\'.
	self json: '""' equals: ''.! !

!JsonTests methodsFor: 'as yet unclassified' stamp: 'tonyg 8/16/2005 23:36'!
testAtomTrue
	self json: 'true' equals: true.
	self json: '  true' equals: true.
	self json: 'true  ' equals: true.
	self json: '  true  ' equals: true.
! !

!JsonTests methodsFor: 'as yet unclassified' stamp: 'tonyg 8/17/2005 00:00'!
testDictionary
	self json: '{}' equals: (Dictionary new).
	self json: '{"a": "a"}' equals: (Dictionary new at: 'a' put: 'a'; yourself).
	self json: '{"a": [[]]}' equals: (Dictionary new at: 'a' put: #(#()); yourself).
	self json: '{"a":"b", "b":"a"}' equals: ({'a'->'b'. 'b'->'a'} as: Dictionary).! !

!JsonTests methodsFor: 'as yet unclassified' stamp: 'tonyg 8/17/2005 16:25'!
testWriteString
	self render: '"' equals: '"\""'.
	self render: '\' equals: '"\\"'.
	self render: 'hi' equals: '"hi"'.
	self render: ({$a. Character lf. $b} as: String) equals: '"a\nb"'.! !

Error subclass: #JsonSyntaxError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'JSON'!

!JsonSyntaxError commentStamp: '<historical>' prior: 0!
Class Json signals instances of me when an input stream contains invalid JSON input.!

!UndefinedObject methodsFor: '*JSON-writing' stamp: 'tonyg 8/17/2005 00:45'!
jsonWriteOn: aWriteStream 
	aWriteStream nextPutAll: 'null'! !

!Number methodsFor: '*JSON-writing' stamp: 'tonyg 8/17/2005 00:41'!
jsonWriteOn: aWriteStream 
	aWriteStream nextPutAll: self asString.! !

!String methodsFor: '*JSON-writing' stamp: 'tonyg 8/17/2005 16:21'!
jsonWriteOn: aStream
	| replacement |
	aStream nextPut: $".
	self do: [:ch |
		replacement _ Json escapeForCharacter: ch.
		replacement
			ifNil: [ aStream nextPut: ch ]
			ifNotNil: [ aStream nextPut: $\; nextPut: replacement ].
	].
	aStream nextPut: $".
! !

!SequenceableCollection methodsFor: '*JSON-writing' stamp: 'tonyg 8/17/2005 00:40'!
jsonWriteOn: aStream
	| needComma |
	needComma := false.
	aStream nextPut: $[.
	self do: [:v |
		needComma
			ifTrue: [ aStream nextPutAll: ', ' ]
			ifFalse: [ needComma := true ].
		v jsonWriteOn: aStream.
	].
	aStream nextPut: $].! !

Object subclass: #Json
	instanceVariableNames: 'stream'
	classVariableNames: 'CharacterEscapeMap'
	poolDictionaries: ''
	category: 'JSON'!

!Json commentStamp: '<historical>' prior: 0!
This class reads and writes JSON format data - strings, numbers, boolean, nil, arrays and dictionaries. See http://www.crockford.com/JSON/index.html.!

!Json class methodsFor: 'as yet unclassified' stamp: 'tonyg 8/17/2005 16:20'!
escapeForCharacter: c
	^ CharacterEscapeMap at: c ifAbsent: [nil]! !

!Json class methodsFor: 'as yet unclassified' stamp: 'tonyg 8/17/2005 16:19'!
initialize
	"Json initialize."
	CharacterEscapeMap := Dictionary newFrom: {
		$" -> $".
		$\ -> $\.
		Character backspace -> $b.
		Character lf -> $n.
		Character newPage -> $f.
		Character cr -> $r.
		Character tab -> $t.
	}.! !

!Json class methodsFor: 'as yet unclassified' stamp: 'tonyg 8/17/2005 09:09'!
numbersMayContain: aChar
	^ aChar isDigit or: [#($- $+ $. $e $E) includes: aChar]! !

!Json class methodsFor: 'as yet unclassified' stamp: 'tonyg 8/16/2005 20:19'!
readFrom: aStream
	^ self new stream: aStream; readAny.! !

!Json class methodsFor: 'as yet unclassified' stamp: 'tonyg 8/17/2005 07:45'!
render: anObject
	| s |
	s := WriteStream on: String new.
	anObject jsonWriteOn: s.
	^ s contents.! !

!Json methodsFor: 'private' stamp: 'tonyg 8/16/2005 23:41'!
consume: aString returning: anObject
	aString do: [:c | self next == c ifFalse: [JsonSyntaxError signal: 'Expected ''', aString, ''''] ].
	^ anObject! !

!Json methodsFor: 'private' stamp: 'tonyg 8/16/2005 21:08'!
interpretStringEscape
	| c |
	c := self next.
	c == $b ifTrue: [^ Character backspace].
	c == $n ifTrue: [^ Character lf].
	c == $f ifTrue: [^ Character newPage].
	c == $r ifTrue: [^ Character cr].
	c == $t ifTrue: [^ Character tab].
	^ c.! !

!Json methodsFor: 'private' stamp: 'tonyg 8/16/2005 20:22'!
next
	^ self stream next! !

!Json methodsFor: 'private' stamp: 'tonyg 8/16/2005 20:21'!
peek
	^ self stream peek! !

!Json methodsFor: 'parsing' stamp: 'topa 11/26/2009 11:58'!
readAny
	"This is the main entry point for the JSON parser. See also readFrom: on the class side."
	| c |
	self skipWhitespace.
	c := self peek asLowercase.
	c == ${ ifTrue: [self next. ^ self readDictionary].
	c == $[ ifTrue: [self next. ^ self readArray].
	c == $" ifTrue: [self next. ^ self readString].
	c == $( ifTrue: [self next. ^ self readPoint].
	c == $t ifTrue: [^ self consume: 'true' returning: true].
	c == $f ifTrue: [^ self consume: 'false' returning: false].
	c == $n ifTrue: [^ self consume: 'null' returning: nil].
	(Json numbersMayContain: c) ifTrue: [^ self readNumber].
	JsonSyntaxError signal: 'Unknown Json input'! !

!Json methodsFor: 'private' stamp: 'tonyg 8/16/2005 23:49'!
readArray
	| a needComma |
	a := OrderedCollection new.
	needComma _ false.
	[
		self skipWhitespace.
		self peek == $] ifTrue: [self next. ^ a asArray].
		needComma
			ifTrue: [self peek == $, ifFalse: [JsonSyntaxError signal: 'Missing comma'].
					self next.]
			ifFalse: [needComma _ true]. 
		a add: self readAny.
	] repeat.
! !

!Json methodsFor: 'private' stamp: 'tonyg 8/16/2005 23:55'!
readDictionary
	| m k v needComma |
	m := Dictionary new.
	needComma _ false.
	[
		self skipWhitespace.
		self peek == $} ifTrue: [self next. ^ m].
		needComma
			ifTrue: [self peek == $, ifFalse: [JsonSyntaxError signal: 'Missing comma'].
					self next. self skipWhitespace]
			ifFalse: [needComma _ true.].
		self next == $" ifFalse: [JsonSyntaxError signal: 'Key in dictionary must be string'].
		k := self readString.
		self skipWhitespace.
		self peek == $: ifFalse: [JsonSyntaxError signal: 'Missing colon'].
		self next.
		v := self readAny.
		m at: k put: v.
	] repeat.
! !

!Json methodsFor: 'private' stamp: 'topa 03/26/2009 17:00'!
readNumber
	| acc c |
	acc := WriteStream on: String new.
	[
		c := self peek.
		(c isNil not and: [Json numbersMayContain: c]) ifFalse: [
			[^ acc contents asNumber] on: Error do: [JsonSyntaxError signal: 'Invalid number']].
		acc nextPut: c.
		self next.
	] repeat.! !

!Json methodsFor: 'as yet unclassified' stamp: 'topa 11/26/2009 11:59'!
readPoint

	| x y |
	x := self readNumber.
	self next == $@ ifFalse: [JsonSyntaxError signal: 'Could not read point. At expected.'].
	y := self readNumber.
	self next == $) ifFalse: [JsonSyntaxError signal: 'Point parsing terminated unexpected.'].
	^ x @ y! !

!Json methodsFor: 'private' stamp: 'topa 03/26/2009 17:01'!
readString
	| s c |
	s := WriteStream on: String new.
	[
		c := self next.
		c == $\
			ifTrue: [s nextPut: self interpretStringEscape.]
			ifFalse: [c == $" ifTrue: [^ s contents.].
					s nextPut: c]
	] repeat.! !

!Json methodsFor: 'private' stamp: 'tonyg 8/16/2005 20:32'!
skipComment
	self peek == $/ ifTrue: [
		self next.
		self peek == $/
			ifTrue: [self skipToEndOfLine]
			ifFalse: [self peek == $*
						ifTrue: [self next. self skipCommentBody]
						ifFalse: [JsonSyntaxError signal: 'Invalid comment syntax']]]
! !

!Json methodsFor: 'private' stamp: 'tonyg 8/17/2005 00:19'!
skipCommentBody
	[
		[self next == $*] whileFalse.
		self peek == $/
	] whileFalse.
	self next. "skip that last slash"
	self skipWhitespace.! !

!Json methodsFor: 'private' stamp: 'tonyg 8/16/2005 20:35'!
skipToEndOfLine
	[self peek == Character cr or: [self peek == Character lf]] whileFalse: [self next].
	self skipWhitespace! !

!Json methodsFor: 'private' stamp: 'tonyg 8/16/2005 20:24'!
skipWhitespace
	[self peek isSeparator] whileTrue: [self next].
	self skipComment.! !

!Json methodsFor: 'accessing' stamp: 'tonyg 8/16/2005 20:20'!
stream
	"Answer the value of stream"

	^ stream! !

!Json methodsFor: 'accessing' stamp: 'tonyg 8/16/2005 20:20'!
stream: anObject
	"Set the value of stream"

	stream _ anObject! !

!DateAndTime methodsFor: '*JSON-writing' stamp: 'FHH 7/8/2009 14:57'!
jsonWriteOn: aWriteStream 
	aWriteStream nextPut: $".
	aWriteStream nextPutAll: self asString.
	aWriteStream nextPut: $".! !

!Boolean methodsFor: '*JSON-writing' stamp: 'topa 04/30/2009 09:38'!
jsonWriteOn: aStream
	aStream nextPutAll: self asString! !

!Point methodsFor: '*JSON-writing' stamp: 'topa 11/26/2009 12:12'!
jsonWriteOn: aStream

	aStream
		nextPut: $(;
		nextPutAll: self x asString;
		nextPut: $@;
		nextPutAll: self y asString;
		nextPut: $).
		! !
Json initialize!
